@@ -44,11 +44,17 @@
     (modify-syntax-entry ?<  "." st)
     (modify-syntax-entry ?>  "." st)
 
-    ;; Strings and comments are font-locked separately.
-    (modify-syntax-entry ?\" "." st)
-    (modify-syntax-entry ?\' "." st)
-    (modify-syntax-entry ?`  "." st)
-    (modify-syntax-entry ?\\ "." st)
+    ;; Strings
+    (modify-syntax-entry ?\" "\"" st)
+    (modify-syntax-entry ?\' "\"" st)
+    (modify-syntax-entry ?`  "\"" st)
+    (modify-syntax-entry ?\\ "\\" st)
+
+    ;; Comments
+    (modify-syntax-entry ?/  ". 124b" st)
+    (modify-syntax-entry ?*  ". 23"   st)
+    (modify-syntax-entry ?\n "> b"    st)
+    (modify-syntax-entry ?\^m "> b"   st)
 
     st)
   "Syntax table for Go mode.")
@@ -63,14 +69,12 @@
 some syntax analysis.")
 
 (defvar go-mode-font-lock-keywords
-  (let ((builtins '("append" "cap" "close" "complex" "copy" "delete" "imag" "len"
-                    "make" "new" "panic" "print" "println" "real" "recover"))
+  (let ((builtins '("cap" "close" "closed" "len" "make" "new"
+                    "panic" "panicln" "print" "println"))
         (constants '("nil" "true" "false" "iota"))
         (type-name "\\s *\\(?:[*(]\\s *\\)*\\(?:\\w+\\s *\\.\\s *\\)?\\(\\w+\\)")
         )
-    `((go-mode-font-lock-cs-comment 0 font-lock-comment-face t)
-      (go-mode-font-lock-cs-string 0 font-lock-string-face t)
-      (,(regexp-opt go-mode-keywords 'words) . font-lock-keyword-face)
+    `((,(regexp-opt go-mode-keywords 'words) . font-lock-keyword-face)
       (,(regexp-opt builtins 'words) . font-lock-builtin-face)
       (,(regexp-opt constants 'words) . font-lock-constant-face)
       ;; Function names in declarations
@@ -84,7 +88,7 @@ some syntax analysis.")
       (,(concat "\\<type\\>\\s *\\w+\\s *" type-name) 1 font-lock-type-face)
       ;; Arrays/slices/map value type
       ;; XXX Wrong.  Marks 0 in expression "foo[0] * x"
-      ;;      (,(concat "]" type-name) 1 font-lock-type-face)
+;;      (,(concat "]" type-name) 1 font-lock-type-face)
       ;; Map key type
       (,(concat "\\<map\\s *\\[" type-name) 1 font-lock-type-face)
       ;; Channel value type
@@ -110,7 +114,6 @@ built-ins, functions, and some types.")
   (let ((m (make-sparse-keymap)))
     (define-key m "}" #'go-mode-insert-and-indent)
     (define-key m ")" #'go-mode-insert-and-indent)
-    (define-key m "," #'go-mode-insert-and-indent)
     (define-key m ":" #'go-mode-delayed-electric)
     ;; In case we get : indentation wrong, correct ourselves
     (define-key m "=" #'go-mode-insert-and-indent)
@@ -162,55 +165,27 @@ will be marked from the beginning up to this point (that is, up
 to and including character (1- go-mode-mark-cs-end)).")
 (make-variable-buffer-local 'go-mode-mark-cs-end)
 
-(defvar go-mode-mark-string-end 1
-  "The point at which the string cache ends.  The buffer
-will be marked from the beginning up to this point (that is, up
-to and including character (1- go-mode-mark-string-end)).")
-(make-variable-buffer-local 'go-mode-mark-string-end)
-
-(defvar go-mode-mark-comment-end 1
-  "The point at which the comment cache ends.  The buffer
-will be marked from the beginning up to this point (that is, up
-to and including character (1- go-mode-mark-comment-end)).")
-(make-variable-buffer-local 'go-mode-mark-comment-end)
+(defvar go-mode-mark-cs-state nil
+  "The `parse-partial-sexp' state of the comment/string parser as
+of the point `go-mode-mark-cs-end'.")
+(make-variable-buffer-local 'go-mode-mark-cs-state)
 
 (defvar go-mode-mark-nesting-end 1
   "The point at which the nesting cache ends.  The buffer will be
 marked from the beginning up to this point.")
 (make-variable-buffer-local 'go-mode-mark-nesting-end)
 
-(defun go-mode-mark-clear-cache (b e)
-  "A before-change-function that clears the comment/string and
+(defun go-mode-mark-clear-cache (b e l)
+  "An after-change-function that clears the comment/string and
 nesting caches from the modified point on."
 
   (save-restriction
     (widen)
-    (when (<= b go-mode-mark-cs-end)
-      ;; Remove the property adjacent to the change position.
-      ;; It may contain positions pointing beyond the new end mark.
-      (let ((b (let ((cs (get-text-property (max 1 (1- b)) 'go-mode-cs)))
-		 (if cs (car cs) b))))
-	(remove-text-properties
-	 b (min go-mode-mark-cs-end (point-max)) '(go-mode-cs nil))
-	(setq go-mode-mark-cs-end b)))
+    (when (< b go-mode-mark-cs-end)
+      (remove-text-properties b (min go-mode-mark-cs-end (point-max)) '(go-mode-cs nil))
+      (setq go-mode-mark-cs-end b
+            go-mode-mark-cs-state nil))
 
-    (when (<= b go-mode-mark-string-end)
-      ;; Remove the property adjacent to the change position.
-      ;; It may contain positions pointing beyond the new end mark.
-      (let ((b (let ((cs (get-text-property (max 1 (1- b)) 'go-mode-string)))
-		 (if cs (car cs) b))))
-	(remove-text-properties
-	 b (min go-mode-mark-string-end (point-max)) '(go-mode-string nil))
-	(setq go-mode-mark-string-end b)))
-    (when (<= b go-mode-mark-comment-end)
-      ;; Remove the property adjacent to the change position.
-      ;; It may contain positions pointing beyond the new end mark.
-      (let ((b (let ((cs (get-text-property (max 1 (1- b)) 'go-mode-comment)))
-		 (if cs (car cs) b))))
-	(remove-text-properties
-	 b (min go-mode-mark-string-end (point-max)) '(go-mode-comment nil))
-	(setq go-mode-mark-comment-end b)))
-    
     (when (< b go-mode-mark-nesting-end)
       (remove-text-properties b (min go-mode-mark-nesting-end (point-max)) '(go-mode-nesting nil))
       (setq go-mode-mark-nesting-end b))))
@@ -235,7 +210,7 @@ context-sensitive."
                (progn ,@body)
              (set-buffer-modified-p ,modified-var)))))))
 
-(defun go-mode-cs (&optional pos)
+(defsubst go-mode-cs (&optional pos)
   "Return the comment/string state at point POS.  If point is
 inside a comment or string (including the delimiters), this
 returns a pair (START . END) indicating the extents of the
@@ -243,218 +218,45 @@ comment or string."
 
   (unless pos
     (setq pos (point)))
-  (when (> pos go-mode-mark-cs-end)
-    (go-mode-mark-cs pos))
-  (get-text-property pos 'go-mode-cs))
+  (if (= pos 1)
+      nil
+    (when (> pos go-mode-mark-cs-end)
+      (go-mode-mark-cs pos))
+    (get-text-property (- pos 1) 'go-mode-cs)))
 
 (defun go-mode-mark-cs (end)
   "Mark comments and strings up to point END.  Don't call this
 directly; use `go-mode-cs'."
+
   (setq end (min end (point-max)))
   (go-mode-parser
-   (save-match-data
-     (let ((pos
-	    ;; Back up to the last known state.
-	    (let ((last-cs
-		   (and (> go-mode-mark-cs-end 1)
-			(get-text-property (1- go-mode-mark-cs-end) 
-					   'go-mode-cs))))
-	      (if last-cs
-		  (car last-cs)
-		(max 1 (1- go-mode-mark-cs-end))))))
-       (while (< pos end)
-	 (goto-char pos)
-	 (let ((cs-end			; end of the text property
-		(cond
-		 ((looking-at "//")
-		  (end-of-line)
-		  (1+ (point)))
-		 ((looking-at "/\\*")
-		  (goto-char (+ pos 2))
-		  (if (search-forward "*/" (1+ end) t)
-		      (point)
-		    end))
-		 ((looking-at "\"")
-		  (goto-char (1+ pos))
-		  (if (looking-at "[^\"\n\\\\]*\\(\\\\.[^\"\n\\\\]*\\)*\"")
-		      (match-end 0)
-		    (end-of-line)
-		    (point)))
-		 ((looking-at "'")
-		  (goto-char (1+ pos))
-		  (if (looking-at "[^'\n\\\\]*\\(\\\\.[^'\n\\\\]*\\)*'")
-		      (match-end 0)
-		    (end-of-line)
-		    (point)))
-		 ((looking-at "`")
-		  (goto-char (1+ pos))
-		  (while (if (search-forward "`" end t)
-			     (if (eq (char-after) ?`)
-				 (goto-char (1+ (point))))
-			   (goto-char end)
-			   nil))
-		  (point)))))
-	   (cond
-	    (cs-end
-	     (put-text-property pos cs-end 'go-mode-cs (cons pos cs-end))
-	     (setq pos cs-end))
-	    ((re-search-forward "[\"'`]\\|/[/*]" end t)
-	     (setq pos (match-beginning 0)))
-	    (t
-	     (setq pos end)))))
-       (setq go-mode-mark-cs-end pos)))))
-
-(defun go-mode-in-comment (&optional pos)
-  "Return the comment/string state at point POS.  If point is
-inside a comment (including the delimiters), this
-returns a pair (START . END) indicating the extents of the
-comment or string."
-
-  (unless pos
-    (setq pos (point)))
-  (when (> pos go-mode-mark-comment-end)
-    (go-mode-mark-comment pos))
-  (get-text-property pos 'go-mode-comment))
-
-(defun go-mode-mark-comment (end)
-  "Mark comments up to point END.  Don't call this directly; use `go-mode-in-comment'."
-  (setq end (min end (point-max)))
-  (go-mode-parser
-   (save-match-data
-     (let ((pos
-	    ;; Back up to the last known state.
-	    (let ((last-comment
-		   (and (> go-mode-mark-comment-end 1)
-			(get-text-property (1- go-mode-mark-comment-end) 
-					   'go-mode-comment))))
-	      (if last-comment
-		  (car last-comment)
-		(max 1 (1- go-mode-mark-comment-end))))))
-       (while (< pos end)
-	 (goto-char pos)
-	 (let ((comment-end			; end of the text property
-		(cond
-		 ((looking-at "//")
-		  (end-of-line)
-		  (1+ (point)))
-		 ((looking-at "/\\*")
-		  (goto-char (+ pos 2))
-		  (if (search-forward "*/" (1+ end) t)
-		      (point)
-		    end)))))
-	   (cond
-	    (comment-end
-	     (put-text-property pos comment-end 'go-mode-comment (cons pos comment-end))
-	     (setq pos comment-end))
-	    ((re-search-forward "/[/*]" end t)
-	     (setq pos (match-beginning 0)))
-	    (t
-	     (setq pos end)))))
-       (setq go-mode-mark-comment-end pos)))))
-
-(defun go-mode-in-string (&optional pos)
-  "Return the string state at point POS.  If point is
-inside a string (including the delimiters), this
-returns a pair (START . END) indicating the extents of the
-comment or string."
-
-  (unless pos
-    (setq pos (point)))
-  (when (> pos go-mode-mark-string-end)
-    (go-mode-mark-string pos))
-  (get-text-property pos 'go-mode-string))
-
-(defun go-mode-mark-string (end)
-  "Mark strings up to point END.  Don't call this
-directly; use `go-mode-in-string'."
-  (setq end (min end (point-max)))
-  (go-mode-parser
-   (save-match-data
-     (let ((pos
-	    ;; Back up to the last known state.
-	    (let ((last-cs
-		   (and (> go-mode-mark-string-end 1)
-			(get-text-property (1- go-mode-mark-string-end) 
-					   'go-mode-string))))
-	      (if last-cs
-		  (car last-cs)
-		(max 1 (1- go-mode-mark-string-end))))))
-       (while (< pos end)
-	 (goto-char pos)
-	 (let ((cs-end			; end of the text property
-		(cond 
-		 ((looking-at "\"")
-		  (goto-char (1+ pos))
-		  (if (looking-at "[^\"\n\\\\]*\\(\\\\.[^\"\n\\\\]*\\)*\"")
-		      (match-end 0)
-		    (end-of-line)
-		    (point)))
-		 ((looking-at "'")
-		  (goto-char (1+ pos))
-		  (if (looking-at "[^'\n\\\\]*\\(\\\\.[^'\n\\\\]*\\)*'")
-		      (match-end 0)
-		    (end-of-line)
-		    (point)))
-		 ((looking-at "`")
-		  (goto-char (1+ pos))
-		  (while (if (search-forward "`" end t)
-			     (if (eq (char-after) ?`)
-				 (goto-char (1+ (point))))
-			   (goto-char end)
-			   nil))
-		  (point)))))
-	   (cond
-	    (cs-end
-	     (put-text-property pos cs-end 'go-mode-string (cons pos cs-end))
-	     (setq pos cs-end))
-	    ((re-search-forward "[\"'`]" end t)
-	     (setq pos (match-beginning 0)))
-	    (t
-	     (setq pos end)))))
-       (setq go-mode-mark-string-end pos)))))
-
-(defun go-mode-font-lock-cs (limit comment)
-  "Helper function for highlighting comment/strings.  If COMMENT is t,
-set match data to the next comment after point, and advance point
-after it.  If COMMENT is nil, use the next string.  Returns nil
-if no further tokens of the type exist."
-  ;; Ensures that `next-single-property-change' below will work properly.
-  (go-mode-cs limit)
-  (let (cs next (result 'scan))
-    (while (eq result 'scan)
-      (if (or (>= (point) limit) (eobp))
-	  (setq result nil)
-	(setq cs (go-mode-cs))
-	(if cs
-	    (if (eq (= (char-after (car cs)) ?/) comment)
-		;; If inside the expected comment/string, highlight it.
-		(progn
-		  ;; If the match includes a "\n", we have a
-		  ;; multi-line construct.  Mark it as such.
-		  (goto-char (car cs))
-		  (when (search-forward "\n" (cdr cs) t)
-		    (put-text-property
-		     (car cs) (cdr cs) 'font-lock-multline t))
-		  (set-match-data (list (car cs) (cdr cs) (current-buffer)))
-		  (goto-char (cdr cs))
-		  (setq result t))
-	      ;; Wrong type.  Look for next comment/string after this one.
-	      (goto-char (cdr cs)))
-	  ;; Not inside comment/string.  Search for next comment/string.
-	  (setq next (next-single-property-change
-		      (point) 'go-mode-cs nil limit))
-	  (if (and next (< next limit))
-	      (goto-char next)
-	    (setq result nil)))))
-    result))
-
-(defun go-mode-font-lock-cs-string (limit)
-  "Font-lock iterator for strings."
-  (go-mode-font-lock-cs limit nil))
-
-(defun go-mode-font-lock-cs-comment (limit)
-  "Font-lock iterator for comments."
-  (go-mode-font-lock-cs limit t))
+   (let* ((pos go-mode-mark-cs-end)
+          (state (or go-mode-mark-cs-state (syntax-ppss pos))))
+     ;; Mark comments and strings
+     (when (nth 8 state)
+       ;; Get to the beginning of the comment/string
+       (setq pos (nth 8 state)
+             state nil))
+     (while (> end pos)
+       ;; Find beginning of comment/string
+       (while (and (> end pos)
+                   (progn
+                     (setq state (parse-partial-sexp pos end nil nil state 'syntax-table)
+                           pos (point))
+                     (not (nth 8 state)))))
+       ;; Find end of comment/string
+       (let ((start (nth 8 state)))
+         (when start
+           (setq state (parse-partial-sexp pos (point-max) nil nil state 'syntax-table)
+                 pos (point))
+           ;; Mark comment
+           (put-text-property start (- pos 1) 'go-mode-cs (cons start pos))
+           (when nil
+             (put-text-property start (- pos 1) 'face
+                                `((:background "midnight blue")))))))
+     ;; Update state
+     (setq go-mode-mark-cs-end   pos
+           go-mode-mark-cs-state state))))
 
 (defsubst go-mode-nesting (&optional pos)
   "Return the nesting at point POS.  The nesting is a list
@@ -544,32 +346,6 @@ token on the line."
        (when (/= (skip-chars-backward "[:word:]_") 0)
          (not (looking-at go-mode-non-terminating-keywords-regexp)))))))
 
-(defun go-mode-whitespace-p (char)
-  "Is char whitespace in the syntax table for go."
-  (eq 32 (char-syntax char)))
-
-(defun go-mode-backward-skip-comments ()
-  "Skip backward over comments and whitespace."
-  ;; only proceed if point is in a comment or white space
-  (if (or (go-mode-in-comment)
-	  (go-mode-whitespace-p (char-after (point))))
-      (let ((loop-guard t))
-	(while (and
-		loop-guard
-		(not (bobp)))
-
-	  (cond ((go-mode-whitespace-p (char-after (point)))
-		 ;; moves point back over any whitespace
-		 (re-search-backward "[^[:space:]]"))
-
-		((go-mode-in-comment)
-		 ;; move point to char preceeding current comment
-		 (goto-char (1- (car (go-mode-in-comment)))))
-		
-		;; not in a comment or whitespace? we must be done.
-		(t (setq loop-guard nil)
-		   (forward-char 1)))))))
-
 (defun go-mode-indentation ()
   "Compute the ideal indentation level of the current line.
 
@@ -615,10 +391,9 @@ indented one level."
                    (incf indent tab-width))
                   ((?\()
                    (goto-char (car nest))
-                   (go-mode-backward-skip-comments)
-                   (backward-char)
+                   (forward-comment (- (buffer-size)))
                    ;; Really just want the token before
-                   (when (looking-back "\\<import\\|const\\|var\\|type\\|package"
+                   (when (looking-back "\\<import\\|const\\|var\\|type"
                                        (max (- (point) 7) (point-min)))
                      (incf indent tab-width)
                      (when first
@@ -629,13 +404,8 @@ indented one level."
           (when (looking-at "\\<case\\>\\|\\<default\\>\\|\\w+\\s *:\\(\\S.\\|$\\)")
             (decf indent tab-width))
 
-	  (when (looking-at "\\w+\\s *:.+,\\s *$")
-	    (incf indent tab-width))
-
           ;; Continuation lines are indented 1 level
-          (beginning-of-line)		; back up to end of previous line
-	  (backward-char)
-          (go-mode-backward-skip-comments) ; back up past any comments
+          (forward-comment (- (buffer-size)))
           (when (case (char-before)
                   ((nil ?\{ ?:)
                    ;; At the beginning of a block or the statement
@@ -669,15 +439,12 @@ indented one level."
   "Indent the current line according to `go-mode-indentation'."
   (interactive)
 
-  ;; turn off case folding to distinguish keywords from identifiers
-  ;; e.g. "default" is a keyword; "Default" can be a variable name.
-  (let ((case-fold-search nil))
-    (let ((col (go-mode-indentation)))
-      (when col
-	(let ((offset (- (current-column) (current-indentation))))
-	  (indent-line-to col)
-	  (when (> offset 0)
-	    (forward-char offset)))))))
+  (let ((col (go-mode-indentation)))
+    (when col
+      (let ((offset (- (current-column) (current-indentation))))
+        (indent-line-to col)
+        (when (> offset 0)
+          (forward-char offset))))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;; Go mode
@@ -703,8 +470,9 @@ functions, and some types.  It also provides indentation that is
 
   ;; Reset the syntax mark caches
   (setq go-mode-mark-cs-end      1
+        go-mode-mark-cs-state    nil
         go-mode-mark-nesting-end 1)
-  (add-hook 'before-change-functions #'go-mode-mark-clear-cache nil t)
+  (add-hook 'after-change-functions #'go-mode-mark-clear-cache nil t)
 
   ;; Indentation
   (set (make-local-variable 'indent-line-function)
@@ -732,92 +500,43 @@ Useful for development work."
 
 ;;;###autoload
 (defun gofmt ()
-  "Pipe the current buffer through the external tool `gofmt`.
+ "Pipe the current buffer through the external tool `gofmt`.
 Replace the current buffer on success; display errors on failure."
 
-  (interactive)
-  (let ((currconf (current-window-configuration)))
-    (let ((srcbuf (current-buffer)))
-      (with-temp-buffer
-	(let ((outbuf (current-buffer))
-	      (errbuf (get-buffer-create "*Gofmt Errors*"))
-	      (coding-system-for-read 'utf-8)    ;; use utf-8 with subprocesses
-	      (coding-system-for-write 'utf-8))
-	  (with-current-buffer errbuf (erase-buffer))
-	  (with-current-buffer srcbuf
-	    (save-restriction
-	      (let (deactivate-mark)
-		(widen)
-		(if (= 0 (shell-command-on-region (point-min) (point-max) "gofmt"
-						  outbuf nil errbuf))
-		    ;; restore window config
-		    ;; gofmt succeeded: replace the current buffer with outbuf,
-		    ;; restore the mark and point, and discard errbuf.
-		    (let ((old-mark (mark t)) (old-point (point)))
-		      (set-window-configuration currconf)
-		      (erase-buffer)
-		      (insert-buffer-substring outbuf)
-		      (goto-char (min old-point (point-max)))
-		      (if old-mark (push-mark (min old-mark (point-max)) t))
-		      (kill-buffer errbuf))
+ (interactive)
+ (let ((srcbuf (current-buffer)))
+   (with-temp-buffer
+     (let ((outbuf (current-buffer))
+           (errbuf (get-buffer-create "*Gofmt Errors*")))
+       (with-current-buffer errbuf (erase-buffer))
+       (with-current-buffer srcbuf
+         (save-restriction
+           (let (deactivate-mark)
+             (widen)
+             (if (= 0 (shell-command-on-region (point-min) (point-max) "gofmt"
+                                               outbuf nil errbuf))
+                 ;; gofmt succeeded: replace the current buffer with outbuf,
+                 ;; restore the mark and point, and discard errbuf.
+                 (let ((old-mark (mark t)) (old-point (point)))
+                   (erase-buffer)
+                   (insert-buffer-substring outbuf)
+                   (goto-char (min old-point (point-max)))
+                   (if old-mark (set-mark (min old-mark (point-max))))
+                   (kill-buffer errbuf))
 
-		  ;; gofmt failed: display the errors
-		  (display-buffer errbuf)))))
+               ;; gofmt failed: display the errors
+               (display-buffer errbuf)))))
 
-	  ;; Collapse any window opened on outbuf if shell-command-on-region
-	  ;; displayed it.
-	  (delete-windows-on outbuf))))))
+       ;; Collapse any window opened on outbuf if shell-command-on-region
+       ;; displayed it.
+       (delete-windows-on outbuf)))))
 
 ;;;###autoload
 (defun gofmt-before-save ()
-  "Add this to .emacs to run gofmt on the current buffer when saving:
+ "Add this to .emacs to run gofmt on the current buffer when saving:
  (add-hook 'before-save-hook #'gofmt-before-save)"
 
-  (interactive)
-  (when (eq major-mode 'go-mode) (gofmt)))
-
-(defun godoc-read-query ()
-  "Read a godoc query from the minibuffer."
-  ;; Compute the default query as the symbol under the cursor.
-  ;; TODO: This does the wrong thing for e.g. multipart.NewReader (it only grabs
-  ;; half) but I see no way to disambiguate that from e.g. foobar.SomeMethod.
-  (let* ((bounds (bounds-of-thing-at-point 'symbol))
-         (symbol (if bounds
-                     (buffer-substring-no-properties (car bounds)
-                                                     (cdr bounds)))))
-    (read-string (if symbol
-                     (format "godoc (default %s): " symbol)
-                   "godoc: ")
-                 nil nil symbol)))
-
-(defun godoc-get-buffer (query)
-  "Get an empty buffer for a godoc query."
-  (let* ((buffer-name (concat "*godoc " query "*"))
-         (buffer (get-buffer buffer-name)))
-    ;; Kill the existing buffer if it already exists.
-    (when buffer (kill-buffer buffer))
-    (get-buffer-create buffer-name)))
-
-(defun godoc-buffer-sentinel (proc event)
-  "Sentinel function run when godoc command completes."
-  (with-current-buffer (process-buffer proc)
-    (cond ((string= event "finished\n")  ;; Successful exit.
-           (goto-char (point-min))
-           (display-buffer (current-buffer) 'not-this-window))
-          ((not (= (process-exit-status proc) 0))  ;; Error exit.
-           (let ((output (buffer-string)))
-             (kill-buffer (current-buffer))
-             (message (concat "godoc: " output)))))))
-
-;;;###autoload
-(defun godoc (query)
-  "Show go documentation for a query, much like M-x man."
-  (interactive (list (godoc-read-query)))
-  (unless (string= query "")
-    (set-process-sentinel
-     (start-process-shell-command "godoc" (godoc-get-buffer query)
-                                  (concat "godoc " query))
-     'godoc-buffer-sentinel)
-    nil))
+ (interactive)
+ (when (eq major-mode 'go-mode) (gofmt)))
 
 (provide 'go-mode)
