@@ -930,6 +930,9 @@ cgen(Node *n, Node *nn)
 
 			return;
 		}
+		o = 0;
+		if(REGARG >= 0)
+			o = reg[REGARG];
 		gargs(r, &nod, &nod1);
 		if(l->addable < INDEXED) {
 			reglcgen(&nod, l, nn);
@@ -938,8 +941,9 @@ cgen(Node *n, Node *nn)
 			regfree(&nod);
 		} else
 			gopcode(OFUNC, n->type, Z, l);
-		if(REGARG >= 0 && reg[REGARG])
-			reg[REGARG]--;
+		if(REGARG >= 0)
+			if(o != reg[REGARG])
+				reg[REGARG]--;
 		if(nn != Z) {
 			regret(&nod, n);
 			gmove(&nod, nn);
@@ -1237,12 +1241,11 @@ void
 boolgen(Node *n, int true, Node *nn)
 {
 	int o;
-	Prog *p1, *p2, *p3;
+	Prog *p1, *p2;
 	Node *l, *r, nod, nod1;
 	int32 curs;
 
 	if(debug['g']) {
-		print("boolgen %d\n", true);
 		prtree(nn, "boolgen lhs");
 		prtree(n, "boolgen");
 	}
@@ -1354,15 +1357,6 @@ boolgen(Node *n, int true, Node *nn)
 	case OLO:
 	case OLS:
 		o = n->op;
-		if(true && typefd[l->type->etype] && (o == OEQ || o == ONE)) {
-			// Cannot rewrite !(l == r) into l != r with float64; it breaks NaNs.
-			// Jump around instead.
-			boolgen(n, 0, Z);
-			p1 = p;
-			gbranch(OGOTO);
-			patch(p1, pc);
-			goto com;
-		}
 		if(true)
 			o = comrel[relindex(o)];
 		if(l->complex >= FNX && r->complex >= FNX) {
@@ -1377,10 +1371,6 @@ boolgen(Node *n, int true, Node *nn)
 			break;
 		}
 		if(immconst(l)) {
-			// NOTE: Reversing the comparison here is wrong
-			// for floating point ordering comparisons involving NaN,
-			// but we don't have any of those yet so we don't
-			// bother worrying about it.
 			o = invrel[relindex(o)];
 			/* bad, 13 is address of external that becomes constant */
 			if(r->addable < INDEXED || r->addable == 13) {
@@ -1402,11 +1392,10 @@ boolgen(Node *n, int true, Node *nn)
 				cgen(r, &nod1);
 				gopcode(o, l->type, &nod, &nod1);
 				regfree(&nod1);
-			} else {
+			} else
 				gopcode(o, l->type, &nod, r);
-			}
 			regfree(&nod);
-			goto fixfloat;
+			goto com;
 		}
 		regalloc(&nod, r, nn);
 		cgen(r, &nod);
@@ -1421,33 +1410,6 @@ boolgen(Node *n, int true, Node *nn)
 		} else
 			gopcode(o, l->type, l, &nod);
 		regfree(&nod);
-	fixfloat:
-		if(typefd[l->type->etype]) {
-			switch(o) {
-			case OEQ:
-				// Already emitted AJEQ; want AJEQ and AJPC.
-				p1 = p;
-				gbranch(OGOTO);
-				p2 = p;
-				patch(p1, pc);
-				gins(AJPC, Z, Z);
-				patch(p2, pc);
-				break;
-
-			case ONE:
-				// Already emitted AJNE; want AJNE or AJPS.
-				p1 = p;
-				gins(AJPS, Z, Z);
-				p2 = p;
-				gbranch(OGOTO);
-				p3 = p;
-				patch(p1, pc);
-				patch(p2, pc);
-				gbranch(OGOTO);
-				patch(p3, pc);
-				break;
-			}
-		}
 
 	com:
 		if(nn != Z) {
@@ -1582,7 +1544,7 @@ sugen(Node *n, Node *nn, int32 w)
 			nod0.addable = 0;
 			nod0.right = l;
 
-			// prtree(&nod0, "hand craft");
+			/* prtree(&nod0, "hand craft"); /* */
 			cgen(&nod0, Z);
 		}
 		break;
